/*
 * Example of writing a tuple of 10000000 rows with six columns
 * in a ROOT file by using Rio through AIDA.
 */


#include <AIDA/IAnalysisFactory.h>
#include <AIDA/ITreeFactory.h>
#include <AIDA/ITree.h>
#include <AIDA/IHistogramFactory.h>
#include <AIDA/IHistogram1D.h>
#include <AIDA/IHistogram2D.h>
#include <AIDA/IProfile1D.h>
#include <AIDA/ITupleFactory.h>
#include <AIDA/ITuple.h>

#include <Lib/Printer.h>
#include <Lib/Debug.h>
#include <Lib/Time.h>
#include <Lib/smanip.h>
#include <Lib/mmanip.h>
#include <Lib/Arguments.h>

#include <iostream>

//////////////////////////////////////////////////////////////////////////////
int main(
 int aArgc
,char** aArgv
)
//////////////////////////////////////////////////////////////////////////////
//!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!//
{
  Lib::Debug::checkByClass(true);

  Lib::Arguments args(aArgc,aArgv);

  bool help = args.isAnArgument("-h");
  if(help) {
    std::cout << "BatchLab_jec::main :" << std::endl;
    std::cout << "Options : " << std::endl;
    std::cout << "  OS> BatchLab_jec -h" << std::endl;
    std::cout << "   to have help." << std::endl;
    std::cout << "  OS> BatchLab_jec -rows=<number of rows in the tuple>" 
              << std::endl;
    std::cout << "   Default is 100000." << std::endl;
    std::cout << "  OS> BatchLab_jec -store=[memory,xml,root,hbook,ascii,hippodraw]" << std::endl;
    std::cout << "   to store :"
              << "     in memory"
              << "     in an xml file"
              << "     in a file at the ROOT format with BatchLab objects."
              << "     in a file at the ZEBRA format with HBOOK objects."
              << std::endl;
    std::cout << "  OS> BatchLab_jec -store=root -export=root" << std::endl;
    std::cout << "   to store :"
              << "     in a file at the ROOT format with ROOT objects."
              << std::endl;
    return 0;
  }

  Lib::Time begin(Lib::Time::now());

  AIDA::IAnalysisFactory* analysisFactory = AIDA_createAnalysisFactory();
  if(!analysisFactory) {
    std::cout << "BatchLab_jec::main :"
              << " Can't get an AnalysisFactory." << std::endl;
    return 1;
  }

  AIDA::ITreeFactory* treeFactory = analysisFactory->createTreeFactory();
  if(!treeFactory) {
    std::cout << "BatchLab_jec::main :"
              << " Can't get a TreeFactory." << std::endl;
    return 1;
  }

  // Decide of the storage type and options from args :
  std::string store;
  if(!args.find("-store",store)) store = "memory";
  Lib::smanip::tolowercase(store);
  bool compress  = args.isAnArgument("-compress");
  std::string sexport;
  if(!args.find("-export",sexport)) sexport = "BatchLab";
  AIDA::ITree* tree = 0;
  if(store=="memory") {
    tree = treeFactory->create();
  } else {
    if(store=="root") {
      std::string opts = "compress=";
      opts += compress ? "yes" : "no";
      opts += ";export="+sexport;
      tree = treeFactory->create("BatchLab_jec.root","root",false,true,opts);
    } else if(store=="xml") {
      std::string opts = "compress=";
      opts += compress ? "yes" : "no";
      tree = treeFactory->create("BatchLab_jec.aida","xml",false,true,opts);
    } else if(store=="hbook") {
      tree = treeFactory->create("BatchLab_jec.hbook","hbook",false,true);
    } else if(store=="ascii") {
      tree = treeFactory->create("BatchLab_jec.ascii","ascii",false,true);
    } else if(store=="hippodraw") {
      tree = treeFactory->create("BatchLab_jec.tnt","hippodraw",false,true);
    }
  }
  if(!tree) {
    std::cout << "BatchLab_jec::main :"
              << " Can't create the BatchLab_jec tree." << std::endl;
    return 1;
  }

  if(!tree->mkdir("tuples")) {
    std::cout << "BatchLab_jec::main :"
              << " can't make directory \"tuples\"." << std::endl;
    return 1;
  }

  if(!tree->cd("tuples")) {
    std::cout << "BatchLab_jec::main :"
              << " can't cd to \"tuples\"." << std::endl;
    return 1;
  }

  // Get a tuple factory :
  AIDA::ITupleFactory* tupleFactory = 
    analysisFactory->createTupleFactory(*tree);
  if(!tupleFactory) {
    std::cout << "BatchLab_jec::main :"
              << " Can't get a TupleFactory." << std::endl;
    return 1;
  }


  AIDA::ITuple* tuple =  
    tupleFactory->create("tuple","JEC tuple",
      "int i,double d1, double d2,double d3, double d4,double d5, double d6");
  if(!tuple) {
    std::cout << "BatchLab_jec::main :"
              << " Can't create the \"tuple\" tuple." << std::endl;
    return 1;
  }

  unsigned int rows = 100000;
 {std::string s;
  if(args.find("-rows",s)) {
    if(!Lib::smanip::touint(s,rows)) rows = 100000;
  }}

  Lib::Time beginFill = Lib::Time::now();
  //int nevent = 10000;
  double pi = Lib::mmanip::pi();
  int i;
  for(unsigned long count=0;count<rows;count++) {
    i = int(count + 1);
    tuple->fill(0,i);
    tuple->fill(1,  pi*i);
    tuple->fill(2,2*pi*i);
    tuple->fill(3,3*pi*i);
    tuple->fill(4,4*pi*i);
    tuple->fill(5,5*pi*i);
    tuple->fill(6,6*pi*i);
    tuple->addRow();
  }
  Lib::Time endFill = Lib::Time::elapsed(beginFill);
  std::cout << "BatchLab_jec::main :"
            << " fill tuple elapsed time : " 
            << endFill.seconds() << " seconds."
            << std::endl;

  if(!tree->commit()) {
    std::cout << "BatchLab_jec::main :"
              << " can't commit." << std::endl;
    return 1;
  }

  delete tupleFactory;
  delete treeFactory;
  delete analysisFactory; // Will delete the trees.

  Lib::Time end = Lib::Time::elapsed(begin);

  std::cout << "BatchLab_jec::main :"
            << " elapsed time : " 
            << end.seconds() << " seconds."
            << std::endl;

  // A simple check to see if object creation/destruction balance is correct :
  Lib::Printer printer;
  Lib::Debug::balance(printer);

  return 0;
}

